@@ -7,7 +7,7 @@ relvars.py -- Flatland DB relational variable definitions
     those xUML class model diagrams and text descriptions for both the Flatland Application and Tablet domain models.
     See comments, first the Flatland application domain relvars are defined and then the Tablet relvars.
 """
-from sqlalchemy import Table, Column, String, Integer, Boolean, Enum, Float
+from sqlalchemy import Table, Column, Text, String, Integer, Boolean, Enum, Float
 from sqlalchemy import ForeignKey, UniqueConstraint, PrimaryKeyConstraint, ForeignKeyConstraint, CheckConstraint
 
 
@@ -20,16 +20,114 @@ def define(db) -> dict:
     """
     return {
         # Flatland diagram domain
+        'metadata': Table('Metadata', db.MetaData,
+                          Column('Name', Text, nullable=False, primary_key=True),
+                          ),
+        'titleblock_pattern': Table('Title Block Pattern', db.MetaData,
+                                    Column('Name', Text, nullable=False, primary_key=True),
+                                    ),
+        'compartment_box': Table('Compartment Box', db.MetaData,
+                                 Column('ID', Integer, nullable=False),
+                                 Column('Pattern', Text,
+                                        ForeignKey('Title Block Pattern.Name', name='R308_R303'), nullable=False),
+                                 Column('Orientation', Enum('H', 'V', name='enum_Orientation'), nullable=False),
+                                 Column('Distance', Float, nullable=False),
+                                 Column('Up', Integer, nullable=False),
+                                 Column('Down', Integer, nullable=False),
+                                 PrimaryKeyConstraint('ID', 'Pattern', name='I1'),
+                                 ),
+        'data_box': Table('Data Box', db.MetaData,
+                          Column('ID', Integer, nullable=False),
+                          Column('Pattern', Text,
+                                 ForeignKey('Title Block Pattern.Name', name='R308_R313'), nullable=False),
+                          Column('H align', Enum('LEFT', 'CENTER', 'RIGHT', name='enum_HorizAlign'), nullable=False),
+                          Column('V align', Enum('TOP', 'CENTER', 'BOTTOM', name='enum_VertAlign'), nullable=False),
+                          Column('Style', Text, nullable=False),
+                          PrimaryKeyConstraint('ID', 'Pattern', name='I1'),
+                          ),
+        'sheet_size_group': Table('Sheet Size Group', db.MetaData,
+                                  Column('Name', Text, nullable=False, primary_key=True),
+                                  ),
+        'scaled_titleblock': Table('Scaled Title Block', db.MetaData,
+                                   Column('Title block pattern', Text,
+                                          ForeignKey('Title Block Pattern.Name', name='R301_TBP'), nullable=False),
+                                   Column('Sheet size group', Text,
+                                          ForeignKey('Sheet Size Group.Name', name='R301_SSG'), nullable=False),
+                                   Column('Width', Integer, nullable=False),
+                                   Column('Height', Integer, nullable=False),
+                                   Column('Margin H', Integer, nullable=False),
+                                   Column('Margin V', Integer, nullable=False),
+                                   PrimaryKeyConstraint('Title block pattern', 'Sheet size group', name='I1'),
+                                   ),
         'sheet': Table('Sheet', db.MetaData,
-                       Column('Name', String(20), nullable=False, primary_key=True),
+                       Column('Name', Text, nullable=False, primary_key=True),
                        # US or International names and units
                        Column('Group', Enum('us', 'int', name='enum_Group'), nullable=False),
                        # Based on landscape, so Width should be >= Height
                        # Sizes as string since all metric units are integer and some US values are
                        # half sizes such as 8.5".  Converted to numeric on load from DB
                        Column('Height', String(8), nullable=False),
-                       Column('Width', String(8), nullable=False)
+                       Column('Width', String(8), nullable=False),
+                       Column('Size group', String(20),
+                              ForeignKey('Sheet Size Group.Name', name='R316'), nullable=False),
                        ),
+        'frame': Table('Frame', db.MetaData,
+                       Column('Name', Text, nullable=False),
+                       Column('Sheet', Text, ForeignKey('Sheet.Name', name='R300'), nullable=False),
+                       PrimaryKeyConstraint('Name', 'Sheet', name='I1'),
+                       ),
+        'titleblock_placement': Table('Title Block Placement', db.MetaData,
+                                      Column('Frame', Text, nullable=False),
+                                      Column('Sheet', Text, nullable=False),
+                                      Column('Title block pattern', Text, nullable=False),
+                                      Column('Sheet size group', Text, nullable=False),
+                                      Column('X', Integer, nullable=False),
+                                      Column('Y', Integer, nullable=False),
+                                      PrimaryKeyConstraint('Frame', 'Sheet', name='I1'),
+                                      ForeignKeyConstraint(('Frame', 'Sheet'), ['Frame.Name', 'Frame.Sheet'],
+                                                           name='R315_F'),
+                                      ForeignKeyConstraint(('Title block pattern', 'Sheet size group'),
+                                                           ['Scaled Title Block.Title block pattern',
+                                                            'Scaled Title Block.Sheet size group'], name='R315_STB'),
+                                      ),
+        'box_placement': Table('Box Placement', db.MetaData,
+                               Column('Frame', Text, nullable=False),
+                               Column('Sheet', Text, nullable=False),
+                               Column('Title block pattern', Text, nullable=False),
+                               Column('Box', Integer, nullable=False),
+                               Column('X', Integer, nullable=False),
+                               Column('Y', Integer, nullable=False),
+                               Column('Width', Float, nullable=False),
+                               Column('Height', Float, nullable=False),
+                               PrimaryKeyConstraint('Frame', 'Sheet', 'Title block pattern', 'Box', name='I1'),
+                               # Constraint below keeps failing for some reason so it is commented out for now
+                               # And, yes, I have checked EVERYTHING.  It is a total mystery
+                               # ForeignKeyConstraint(('Frame', 'Sheet', 'Title block pattern',),
+                               #     ['Title Block Placement.Frame', 'Title Block Placement.Sheet'], name='R318_TBP'),
+                               ),
+        'box_text_line': Table('Box Text Line', db.MetaData,
+                               Column('Metadata', Text, ForeignKey('Metadata.Name', name='R319_M'),
+                                      nullable=False),
+                               Column('Box', Integer, nullable=False),
+                               Column('Title block pattern', Text,
+                                      ForeignKey('Title Block Pattern.Name', name='R319_R308_R303'),
+                                      nullable=False),
+                               Column('Order', Integer, nullable=False),
+                               PrimaryKeyConstraint('Metadata', 'Box', 'Title block pattern', name='I1'),
+                               UniqueConstraint('Box', 'Title block pattern', 'Order', name='I2'),
+                               ),
+        'open_field': Table('Open Field', db.MetaData,
+                            Column('Metadata', Text, ForeignKey('Metadata.Name', name='R305_R307_M'), nullable=False),
+                            Column('Frame', Text, nullable=False),
+                            Column('Sheet', Text, nullable=False),
+                            Column('x position', Integer, nullable=False),
+                            Column('y position', Integer, nullable=False),
+                            Column('max width', Integer, nullable=False),
+                            Column('max height', Integer, nullable=False),
+                            PrimaryKeyConstraint('Metadata', 'Frame', 'Sheet', name='I1'),
+                            ForeignKeyConstraint(('Frame', 'Sheet'), ['Frame.Name', 'Frame.Sheet'],
+                                                 name='R305_R307_F'),
+                            ),
         'diagram_layout_specification': Table('Diagram Layout Specification', db.MetaData,
                                               Column('Name', String(20), nullable=False, primary_key=True),
                                               Column('Default margin top', Integer, nullable=False),
@@ -113,7 +211,7 @@ def define(db) -> dict:
         'nameable_connector_location': Table('Nameable Connector Location', db.MetaData,
                                              Column('Name', String, nullable=False),
                                              Column('Diagram type', String,
-                                                    ForeignKey('Diagram Type.Name',name='UR70'), nullable=False),
+                                                    ForeignKey('Diagram Type.Name', name='UR70'), nullable=False),
                                              PrimaryKeyConstraint('Name', 'Diagram type', name='I1'),
                                              ),
         'connector_type': Table('Connector Type', db.MetaData,
