@@ -10,7 +10,7 @@ from flatland.input.nocomment import nocomment
 import os
 from pathlib import Path
 
-Subsystem = namedtuple('Subsystem', 'name classes rels')
+Subsystem = namedtuple('Subsystem', 'name classes rels metadata')
 
 class ModelParser:
     """
@@ -87,11 +87,15 @@ class ModelParser:
             peg_tree_dot.unlink(missing_ok=True)
             peg_model_dot.unlink(missing_ok=True)
         # Return the refined model data, checking sequence length
-        subsys_name = result[0]  # Required by model parser
-        class_data = result[1]  # Required by model parser
-        rel_data = None if len(result) < 3 else result[2]  # Optional
+        metadata = result.results.get('metadata', None)  # Optional section
+        subsys_name = result.results['subsystem_header'][0]  # Required by model parser
+        class_data = result.results['class_set'][0]  # Required by model parser
+        rel_data = result.results.get('rel_section', None)  # Optional section
         # You can draw classes without rels, but not the other way around!
-        return Subsystem(name=subsys_name, classes=class_data, rels=rel_data)
+        return Subsystem(
+            name=subsys_name, classes=class_data, rels=None if not rel_data else rel_data[0],
+            metadata=None if not metadata else metadata[0]
+        )
 
 
 if __name__ == "__main__":
